package cn.ibizlab.util.repository;

import cn.ibizlab.util.filter.MongoQueryContext;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.repository.query.MongoEntityInformation;
import org.springframework.data.mongodb.repository.support.SimpleMongoRepository;
import org.springframework.data.support.PageableExecutionUtils;

import java.util.List;

public class IBZMongoSimpleRepository<T, ID> extends SimpleMongoRepository<T, ID> implements IBZMongoRepository<T, ID> {

    private final MongoOperations mongoOperations;
    private final MongoEntityInformation<T, ID> entityInformation;

    public IBZMongoSimpleRepository(MongoEntityInformation<T, ID> metadata, MongoOperations mongoOperations) {
        super(metadata, mongoOperations);
        this.entityInformation = metadata;
        this.mongoOperations = mongoOperations;
    }

    @Override
    public <S extends T> Page<T> query(MongoQueryContext mongoQueryContext) {
        Query query = mongoQueryContext.getSearchCond();
            query.with(mongoQueryContext.getPageable());
        if(mongoQueryContext.getPageSort()!=null)
            query.with(mongoQueryContext.getPageSort());
        List<T> list = mongoOperations.find(query, entityInformation.getJavaType(), entityInformation.getCollectionName());

        return PageableExecutionUtils.getPage(list, mongoQueryContext.getPageable(),
                () -> mongoOperations.count(query, entityInformation.getJavaType(), entityInformation.getCollectionName()));
    }
}
